import { Switch, Route, useLocation } from "wouter";
import { queryClient } from "./lib/queryClient";
import { QueryClientProvider } from "@tanstack/react-query";
import { Toaster } from "@/components/ui/toaster";
import { TooltipProvider } from "@/components/ui/tooltip";
import { motion, AnimatePresence } from "framer-motion";
import { Suspense, lazy, useEffect } from "react";
import { preloadCriticalComponents } from "./lib/preloader";
import { Navbar } from "@/components/layout/Navbar";
import { Footer } from "@/components/layout/Footer";
import { BookPageWrapper, FloatingBookElements } from "@/components/transitions/PageTransition";
import { PageTurnSounds } from "@/components/transitions/PageTurnSounds";

// Import lightweight components directly
import NotFound from "@/pages/not-found";
import Home from "@/pages/home";
import Register from "@/pages/register";
import Login from "@/pages/login";
import Discovery from "@/pages/discovery"; // Load discovery immediately

// Lazy load other heavy components
const BookMap = lazy(() => import("@/pages/book-map"));
const Journey = lazy(() => import("@/pages/journey"));
const Bookshelf = lazy(() => import("@/pages/bookshelf"));
const GenreDiscovery = lazy(() => import("@/pages/genre-discovery"));
const BookDetails = lazy(() => import("@/pages/book-details"));
const ReadingListPage = lazy(() => import("@/pages/reading-list"));
const SavedMapsPage = lazy(() => import("@/pages/saved-maps"));
const SharedJourneyPage = lazy(() => import("@/pages/shared-journey"));
const ReadingGoalsPage = lazy(() => import("@/components/goals/ReadingGoalsPage").then(m => ({ default: m.ReadingGoalsPage })));
const ReadingGoalPage = lazy(() => import("@/pages/reading-goal"));
const RecommendationsPage = lazy(() => import("@/pages/recommendations"));

// Loading component for better UX
function PageLoader() {
  return (
    <div className="min-h-screen flex items-center justify-center">
      <motion.div
        className="flex flex-col items-center gap-4"
        initial={{ opacity: 0 }}
        animate={{ opacity: 1 }}
        transition={{ duration: 0.3 }}
      >
        <motion.div
          className="w-12 h-12 border-4 border-primary/20 border-t-primary rounded-full"
          animate={{ rotate: 360 }}
          transition={{ duration: 1, repeat: Infinity, ease: "linear" }}
        />
        <p className="text-sm text-gray-600">Loading page...</p>
      </motion.div>
    </div>
  );
}

function Router() {
  const [location] = useLocation();

  return (
    <div className="min-h-screen">
      {/* Paper background effect */}
      <div className="absolute inset-0 bg-gradient-to-br from-[#FFFAF5] via-[#FFF8F0] to-[#F5F0E8] opacity-30" />

      <div className="relative z-10">
        <Suspense fallback={<PageLoader />}>
          <Switch>
            <Route path="/" component={Home} />
            <Route path="/register" component={Register} />
            <Route path="/login" component={Login} />
            <Route path="/journey/:id" component={Journey} />
            <Route path="/journey" component={Journey} />
            <Route path="/book/:id" component={BookDetails} />
            <Route path="/paths" component={Journey} />
            <Route path="/book-map" component={BookMap} />
            <Route path="/discovery">
              {() => <Discovery key="discovery" />}
            </Route>
            <Route path="/bookshelf" component={Bookshelf} />
            <Route path="/reading-list" component={ReadingListPage} />
            <Route path="/saved-maps" component={SavedMapsPage} />
            <Route path="/goals" component={ReadingGoalsPage} />
            <Route path="/reading-goals" component={ReadingGoalsPage} />
            <Route path="/reading-goal" component={ReadingGoalPage} />
            <Route path="/recommendations" component={RecommendationsPage} />
            <Route path="/books/surprise">
              {() => <Discovery key="surprise" />}
            </Route>
            <Route path="/genre-discovery" component={GenreDiscovery} />
            <Route path="/shared-journey/:id" component={SharedJourneyPage} />
            <Route component={NotFound} />
          </Switch>
        </Suspense>
      </div>
    </div>
  );
}

function App() {
  // Preload critical components on app start
  useEffect(() => {
    // Small delay to not block initial render
    const timer = setTimeout(() => {
      preloadCriticalComponents();
    }, 100);

    return () => clearTimeout(timer);
  }, []);

  return (
    <QueryClientProvider client={queryClient}>
      <TooltipProvider>
        <div className="min-h-screen flex flex-col paper-texture relative overflow-hidden">
          {/* Enhanced background effects */}
          <div className="absolute inset-0 overflow-hidden pointer-events-none">
            <div className="absolute -top-40 -right-40 w-80 h-80 bg-gradient-to-br from-accent/10 to-transparent rounded-full blur-3xl" />
            <div className="absolute -bottom-40 -left-40 w-80 h-80 bg-gradient-to-tr from-secondary/10 to-transparent rounded-full blur-3xl" />
            <div className="absolute top-1/2 left-1/2 transform -translate-x-1/2 -translate-y-1/2 w-96 h-96 bg-gradient-to-r from-primary/5 to-transparent rounded-full blur-3xl" />
          </div>

          {/* Floating book elements for ambiance */}
          <FloatingBookElements />

          {/* Enhanced book spine on the left */}
          <motion.div
            className="fixed left-0 top-0 w-6 h-full z-40 overflow-hidden"
            initial={{ x: -24 }}
            animate={{ x: 0 }}
            transition={{ duration: 0.8, delay: 0.2 }}
          >
            {/* Main spine gradient */}
            <div className="absolute inset-0 bg-gradient-to-b from-amber-800 via-amber-700 to-amber-900" />

            {/* Leather texture overlay */}
            <div 
              className="absolute inset-0 opacity-20"
              style={{
                backgroundImage: `url("data:image/svg+xml,%3Csvg xmlns='http://www.w3.org/2000/svg' width='4' height='4' viewBox='0 0 4 4'%3E%3Cpath fill='%23000' fill-opacity='0.4' d='M1 3h1v1H1V3zm2-2h1v1H3V1z'%3E%3C/path%3E%3C/svg%3E")`,
              }}
            />

            {/* Enhanced spine details */}
            <div className="absolute inset-y-0 left-1 w-0.5 bg-gradient-to-b from-amber-600 to-amber-800" />
            <div className="absolute inset-y-0 right-1 w-0.5 bg-gradient-to-b from-amber-900 to-amber-950" />

            {/* Spine text with glow effect */}
            <div className="absolute top-1/2 left-1/2 transform -translate-x-1/2 -translate-y-1/2 text-amber-100 text-xs font-bold tracking-wider transform -rotate-90 whitespace-nowrap">
              <span className="drop-shadow-sm">WEND</span>
            </div>

            {/* Subtle shadow for depth */}
            <div className="absolute right-0 top-0 w-2 h-full bg-gradient-to-r from-transparent to-black/10" />
          </motion.div>

          {/* Main content with margin for spine */}
          <div className="ml-6 relative z-10">
            <Navbar />
            <main className="flex-grow">
              <Router />
            </main>
            <Footer />
          </div>

          {/* Page turn sound effects - disabled */}
          {/* <PageTurnSounds /> */}
        </div>
        <Toaster />
      </TooltipProvider>
    </QueryClientProvider>
  );
}

export default App;